const debug = require('debug');

const LEVELS = [ 'silent', 'error', 'warn', 'info', 'log' ];
const LOGS = [ null, console.error, console.warn, console.info, console.log ]; // eslint-disable-line

let logLevel = process.browser ? window.localStorage.loglevel : process.env.LOGLEVEL;
let logRules = extract(logLevel || '*:info');
let defaultLogRules = logRules;

class Logger {
  constructor (ns, id) {
    if (typeof id === 'undefined') {
      id = getNsId(ns);
    }
    let nsLevel = getNSLevel(ns);
    LEVELS.forEach((strLevel, level) => {
      if (!level) return;

      let printedLevel = strLevel.toUpperCase().padEnd(5);

      let log = this[strLevel] = debug(`[${printedLevel}] ${ns}/${id}`);

      log.enabled = level <= nsLevel;
      log.log = LOGS[level].bind(console);
    });
  }
}

function getNSLevel (ns) {
  let rule = logRules.find(({ ns: ruleNs, level }) => {
    if (ruleNs === '*') {
      return true;
    }
    return ns.startsWith(ruleNs);
  });

  return rule ? rule.level : 0;
}

let ids = {};
function getNsId (ns) {
  ids[ns] = ids[ns] || 0;
  return ids[ns]++;
}

function logger (ns, id) {
  return new Logger(ns, id);
}

function extract (ruleString = '') {
  let rules = [];
  ruleString.split(',').forEach(token => {
    if (token) {
      let [ ns, level ] = token.split(':');
      level = LEVELS.indexOf(level);
      rules.push({ ns, level });
    }
  });

  return rules;
}

function set (ruleString) {
  logRules = extract(ruleString);
}

function get () {
  return logRules;
}

function reset () {
  logRules = defaultLogRules;
}

Object.assign(logger, {
  Logger,
  LEVELS,
  reset,
  set,
  get,
  extract,
});

module.exports = logger;
